<?php

namespace Ems\Services;

use Ems\Repository\CertisignRepository;
use Ems\Util\Base64Util;
use Ems\Util\ExpectedException;
use Ems\Util\RequestUtils;
use GuzzleHttp\Client;
use Util\Environment;
use Util\NovoCodigo;
use Ems\Services\ParamOrgService;

class CertisignService {
    /** @var Environment */
    protected $environment;

    /** @var CertisignRepository */
    protected $certisignRepository;

    /** @var ParamOrgService */
    protected $paramOrgService;

    /** @var Client */
    protected $http;

    protected $accountInfo = null;

    public function __construct(
        Environment         $environment,
        CertisignRepository $certisignRepository,
        ParamOrgService     $paramOrgService
    ) {
        $this->environment          = $environment;
        $this->certisignRepository  = $certisignRepository;
        $this->paramOrgService      = $paramOrgService;
    }

    public function getAccountInfo() {
        if ($this->accountInfo !== null) {
            return $this->accountInfo;
        }

        $paramOrg = $this->paramOrgService->get();

        $paramsAreSet = isset($paramOrg['DSTOKENAELETRON'])
            && isset($paramOrg['NMREMAELETRON'])
            && isset($paramOrg['DSEMAILAELETRON'])
            && isset($paramOrg['NRCPFREMAELETRON']);

        if (!$paramsAreSet) {
            return null;
        }

        $accountInfo = [
            'token' => $paramOrg['DSTOKENAELETRON'],
            'name' => $paramOrg['NMREMAELETRON'],
            'email' => $paramOrg['DSEMAILAELETRON'],
            'cpf' => $paramOrg['NRCPFREMAELETRON'],
        ];

        return $accountInfo;
    }

    public function upload(
        string $fileName,
        string $base64
    ): string {
        $accountInfo = $this->getAccountInfo();

        $uri = 'https://api-sbx.portaldeassinaturas.com.br/api/v2/document/upload';

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [
            'fileName' => $fileName,
            'bytes' => $base64,
        ];

        $responseBody = RequestUtils::request($uri, $body, 'POST', $headers);
        $uploadId = $responseBody['uploadId'] ?? null;

        if (!$uploadId) {
            throw new ExpectedException('Não foi possível fazer o upload do arquivo no Certisign!');
        }

        return $uploadId;
    }

    public function delete(
        string $id
    ) {
        $accountInfo = $this->getAccountInfo();

        $uri = "https://api-sbx.portaldeassinaturas.com.br/api/v2/document/delete/$id";

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [];

        $responseBody = RequestUtils::request($uri, $body, 'DELETE', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if (!$code) {
            return;
        }

        throw new ExpectedException("(Certisign) $message");
    }

    public function create(
        array $options
    ) {
        $accountInfo = $this->getAccountInfo();

        $documentName = $options['documentName'] ?? null;
        $uploadId = $options['uploadId'] ?? null;
        $uploadName = $options['uploadId'] ?? null;
        $signers =  $options['signers'] ?? [];
        $eSigners =  $options['eSigners'] ?? [];

        $uri = 'https://api-sbx.portaldeassinaturas.com.br/api/v2/document/create';

        $headers = [
            'Content-Type' => 'application/json',
            'Token' => $accountInfo['token'],
        ];

        $body = [
            'document' => [
                'name' => $documentName,
                'upload' => [
                    'id' => $uploadId,
                    'name' => $uploadName,
                ],
            ],
            'sender' => [
                'name' => $accountInfo['name'],
                'email' => $accountInfo['email'],
            ],
            'signers' => $signers,
            'electronicSigners' => $eSigners,
        ];

        $responseBody = RequestUtils::request($uri, $body, 'POST', $headers);
        $code = $responseBody['code'] ?? null;
        $message = $responseBody['message'] ?? null;

        if (!$code) {
            return $responseBody;
        }

        throw new ExpectedException("(Certisign) $message");
    }
}